namespace Degenz

open System

[<Microsoft.FSharp.Core.AutoOpen>]
module Types =

    [<Measure>]
    type mins

    [<Measure>]
    type GBT

    type HackId =
        | Virus = 0
        | RemoteAccess = 1
        | Worm = 2

    type ShieldId =
        | Firewall = 6
        | Encryption = 7
        | Cypher = 8

    type ItemType =
        | Hack = 0
        | Shield = 1
        | Food = 1

    type ItemAttributes = {
        Sell : bool
        Buy : bool
        Consume : bool
        Drop : bool
    }
    with static member empty = { Sell = false ; Buy = false ; Consume = false ; Drop = false }

    type Item = {
        Id : int
        Name : string
        Price : int<GBT>
        Type : ItemType
        Power : int
        Class : int
        Cooldown : int<mins>
        Attributes : ItemAttributes
    }
    with static member empty =
          { Id = -1
            Name = "None"
            Price = 0<GBT>
            Type = ItemType.Hack
            Power = 0
            Class = -1
            Cooldown = 0<mins>
            Attributes = ItemAttributes.empty }

    type HackResult =
        | Strong
        | Weak

    [<CLIMutable>]
    type DiscordPlayer = { Id: uint64; Name: string }
        with static member empty = { Id = 0uL ; Name = "None" }

    type HackEvent = {
        IsInstigator : bool
        Adversary : DiscordPlayer
        Success : bool
        HackId : int
    }

    type PlayerEventType =
        | Hacking of HackEvent
        | Shielding of shieldId : int
        | Stealing of instigator : bool * adversary : DiscordPlayer
        | Imprison

    type PlayerEvent =
        { Type : PlayerEventType
          Cooldown : int<mins>
          Timestamp : DateTime }

    type PlayerTraits = {
        Strength : int
        Focus : int
        Luck : int
        Charisma : int
    }
    with static member empty = { Strength = 0 ; Focus = 0 ; Luck = 0 ; Charisma = 0 }

    [<CLIMutable>]
    type PlayerData = {
        DiscordId : uint64
        Name : string
        Inventory : Item array
        Events : PlayerEvent array
        Traits : PlayerTraits
        Bank : int<GBT>
    }
//        Achievements : string array
//        XP : int
    with member this.basicPlayer = { Id = this.DiscordId ; Name = this.Name }
         static member empty =
             { DiscordId = 0uL
               Name = "None"
               Inventory = [||]
               Events = [||]
               Traits = PlayerTraits.empty
//               Achievements = [||]
//               XP = 0
               Bank = 0<GBT> }
